# Copyright (c) 2012 Canonical Ltd
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program. If not, see
# <http://www.gnu.org/licenses/>.

import datetime
import uuid

import pycassa
from pycassa.system_manager import (
    LEXICAL_UUID_TYPE,
    SystemManager,
    TIME_UUID_TYPE,
    )

from grackle.server.cassandra import workaround_1779


def create_schema(host, keyspace, clobber=False):
    mgr = SystemManager(host)

    if clobber:
        for cf in mgr.get_keyspace_column_families(keyspace):
            mgr.drop_column_family(keyspace, cf)

    try:
        workaround_1779(
            mgr.create_column_family, keyspace, 'message',
            key_validation_class=LEXICAL_UUID_TYPE)
        workaround_1779(
            mgr.create_column_family, keyspace, 'archive_message',
            comparator_type=TIME_UUID_TYPE,
            default_validation_class=LEXICAL_UUID_TYPE)
        pass
    finally:
        mgr.close()


class CassandraConnection(object):

    def __init__(self, keyspace, host):
        self._keyspace = keyspace
        self._host = host
        self._connection = self._connect()
        self.messages = self._column_family('message')
        self.archive_messages = self._column_family('archive_message')

    def _connect(self):
        return pycassa.connect(self._keyspace, self._host)

    def _column_family(self, name):
        return pycassa.ColumnFamily(self._connection, name)

    def add_message(self, archive_uuid, message):
        message_uuid = uuid.uuid4()
        self.messages.insert(
            message_uuid,
            {'date_created': datetime.datetime.utcnow().isoformat() + 'Z',
             'content': message,
             })
        self.archive_messages.insert(
            archive_uuid, {datetime.datetime.now(): message_uuid})
        return message_uuid

    def get_messages(self, archive_uuid):
        ids = self.archive_messages.get(
            archive_uuid, column_count=10, column_reversed=True).values()
        messages = self.messages.multiget(ids)
        return [messages[id]['content'] for id in ids]
